@@ -21,7 +21,8 @@ import json
 logging.getLogger('tensorflow').setLevel(logging.ERROR)  # suppress warnings
 os.system('clear')
 
-def get_ASTROMER(num_layers=2,
+class ASTROMER:
+    def __init__(self, num_layers=2,
                  d_model=200,
                  num_heads=2,
                  dff=256,
@@ -32,70 +33,45 @@ def get_ASTROMER(num_layers=2,
                  maxlen=100,
                  batch_size=None,
                  pretrained_weights=False,
+                 finetuning = None,
                  overwrite=False):
-    if pretrained_weights:
-        #model = load_weights(git_link="https://github.com/HarshVardhanGoyal/test_model.git", model_name="astromer_10022021")
-        model_name="astromer_10022021"
-        git_link="https://github.com/HarshVardhanGoyal/test_model.git"
-        model_path = os.path.join(os.getcwd(),"weights")
-    
-        try: 
-            test_dir = tempfile.mkdtemp()
-            git.Repo.clone_from(git_link, test_dir, branch='main', depth=1)
-            time.sleep(2)
-            if os.path.path.exists(model_path):
-                if overwrite:
-                    #shutil.rmtree(model_path)
-                    pass
-                else:
-                    print("The saved weights already exists")
-        except:
-            pass
-        finally:
-            sourcepath = os.path.join(test_dir, model_name)
-            sourcefiles = os.listdir(sourcepath)
-            destinationpath = os.path.join(model_path, model_name)
-            os.makedirs(model_path)
-            # for file in sourcefiles:
-            #     if file.endswith('.index') or file.endswith('.data-00000-of-00001') or file.endswith('.json') or file.endswith("checkpoint"):
-            #         print(sourcefiles)
-            #         print(destinationpath)
-            #         print(file)
-            #         shutil.move(os.path.join(sourcepath,file), os.path.join(destinationpath,file))
+        self.num_layers=num_layers
+        self.d_model=d_model
+        self.num_heads=num_heads
+        self.dff=dff
+        self.base=base
+        self.dropout=dropout
+        self.use_leak=use_leak
+        self.no_train=no_train
+        self.maxlen=maxlen
+        self.batch_size=batch_size
+        self.pretrained_weights=pretrained_weights
+        self.overwrite_weights=overwrite
+        self.git_link="https://github.com/HarshVardhanGoyal/test_model.git"
+        self.model_path = os.path.join(os.getcwd(),"weights")
+        self.model_name = "astromer_10022021"
 
-        shutil.move(os.path.join(test_dir, model_name),model_path)
-        # shutil.rmtree(test_dir)
+        # Getting the desired model_name
+        if finetuning in [None, "ogle", "alcock"]:
+            self.finetuning = finetuning
+        else:
+            raise ValueError("Specified fine-tuned model does not exist")
+        
+    # Function to get the model
+    def get_ASTROMER(self):
+        if self.pretrained_weights:
+            self.load_configuration()
 
-        conf_file = os.path.join( model_path, model_name, 'conf.json')
-        with open(conf_file, 'r') as handle:
-            conf = json.load(handle)
-
-        model = get_ASTROMER(num_layers=conf['layers'],
-                            d_model   =conf['head_dim'],
-                            num_heads =conf['heads'],
-                            dff       =conf['dff'],
-                            base      =conf['base'],
-                            dropout   =conf['dropout'],
-                            maxlen    =conf['max_obs'],
-                            use_leak  =conf['use_leak'])
-
-        # Loading the model
-        if os.path.isdir(model_path):
-            model.load_weights(os.path.join(destinationpath, "weights"))
-            print("Weights loaded successfully!")
-
-
-    else:
-        serie  = Input(shape=(maxlen, 1),
+        serie  = Input(shape=(self.maxlen, 1),
                     batch_size=None,
                     name='input')
-        times  = Input(shape=(maxlen, 1),
+        times  = Input(shape=(self.maxlen, 1),
                     batch_size=None,
                     name='times')
-        mask   = Input(shape=(maxlen, 1),
+        mask   = Input(shape=(self.maxlen, 1),
                     batch_size=None,
                     name='mask')
-        length = Input(shape=(maxlen,),
+        length = Input(shape=(self.maxlen,),
                     batch_size=None,
                     dtype=tf.int32,
                     name='length')
@@ -105,16 +81,16 @@ def get_ASTROMER(num_layers=2,
                     'times':times,
                     'length':length}
 
-        encoder = Encoder(num_layers,
-                    d_model,
-                    num_heads,
-                    dff,
-                    base=base,
-                    rate=dropout,
-                    use_leak=use_leak,
+        encoder = Encoder(self.num_layers,
+                    self.d_model,
+                    self.num_heads,
+                    self.dff,
+                    base=self.base,
+                    rate=self.dropout,
+                    use_leak=self.use_leak,
                     name='encoder')
 
-        if no_train:
+        if self.no_train:
             encoder.trainable = False
 
         x = encoder(placeholder)
@@ -124,9 +100,66 @@ def get_ASTROMER(num_layers=2,
                     outputs=x,
                     name="ASTROMER")
 
-    
+        if self.pretrained_weights:
+            # Loading the model weights
+            try:
+                if self.finetuning is not None:
+                    print(f"Loading {self.finetuning} finetuned Model...")
+                    model.load_weights(os.path.join(self.model_path, self.model_name, "finetuning", self.finetuning, "weights"))
+                else:
+                    print("Loading Base Model...")
+                    model.load_weights(os.path.join(self.model_path, self.model_name, "weights"))
+                print("Weights loaded successfully!")
+            except:
+                print("Couldn't load model weights!") 
+        return model
+
+    def load_configuration(self):
+
+        # model_name="astromer_10022021"
+        # git_link="https://github.com/HarshVardhanGoyal/test_model.git"
+        # model_path = os.path.join(os.getcwd(),"weights")
+
+        # Creating a temporary directory and cloning the desired repo
+        test_dir = tempfile.mkdtemp()
+        git.Repo.clone_from(self.git_link, test_dir, branch='main', depth=1)
+        # time.sleep(2)
+
+        # Defining the source path of the weights (i.e temporary directory)
+        sourcepath = os.path.join(test_dir, self.model_name)
+
+        # Checking the directories
+        if not os.path.exists(self.model_path):
+            os.makedirs(self.model_path)
+        
+        if os.path.exists(os.path.join(self.model_path, self.model_name)):
+            if self.overwrite_weights:
+                shutil.rmtree(os.path.join(self.model_path, self.model_name))
+                shutil.move(sourcepath,self.model_path)
+            else:
+                print("The saved weights already exists")
+        else:
+            # Moving the weights to the code directory
+            shutil.move(sourcepath,self.model_path)
+     
+        # Removing the temporary directory
+        os.system('rmdir /S /Q "{}"'.format(test_dir))
+
+        # Loading the conf file from the downloaded weights folder
+        conf_file = os.path.join( self.model_path, self.model_name, 'conf.json')
+        with open(conf_file, 'r') as handle:
+            conf = json.load(handle)
+
+        # Getting the astromer model as per the desired configuration
+        self.num_layers=conf['layers']
+        self.d_model   =conf['head_dim']
+        self.num_heads =conf['heads']
+        self.dff       =conf['dff']
+        self.base      =conf['base']
+        self.dropout   =conf['dropout']
+        self.maxlen    =conf['max_obs']
+        self.use_leak  =conf['use_leak']
         
-    return model
 
 
 @tf.function
